close all;
clear all;

scale = 0.5; % scale of the input image 
k = 5; % number of colors 

% read image 
img = imread('247085.jpg');
im = double(imresize( img, scale )) / 255.0;
[ny,nx,~] = size(im);
n = ny * nx; % number of pixels

% compute colors using kmeans 
[~, C] = kmeans(reshape(im, [n, 3]), k);

% compute data term 
f = zeros(ny, nx, k);
for i=1:k
    f(:, :, i) = sqrt(sum((im - repmat(reshape(C(i, :), 1, 1, 3), ny, nx)).^2, 3));
end
f = reshape(f, [n, k])';

% set up gradient operator D
e = ones(n, 1);
Dx_tilde = spdiags([e -e], -1:0, nx, nx); 
Dx_tilde(:, end) = 0;
e = ones(nx, 1);
Dy_tilde = spdiags([-e e], 0:1, ny, ny);
Dy_tilde(end, end) = 0;
Dx = kron(Dx_tilde', speye(ny));
Dy = kron(speye(nx), Dy_tilde);
D = [Dx; Dy];

% run gradient descent algorithm 
epsilon = 0.1;
tau = 0.25;
alpha = 0.1;
tol = 1e-6;
max_iters = 10000;

u = zeros(k, n);
grad = zeros(k, n);

for it=1:max_iters       
    % compute gradient 
    for i=1:k
        Du = D*(u(i, :)');
        TV = sqrt(Du.^2 + epsilon);
        grad(i,:) = alpha * D'*(Du./TV);
    end
    grad = grad + f;
    
    % perform step
    u_prev = u;
    u = u - tau * grad;
    
    % perform projection 
    u = projSimplex(u')';
    
    % check termination
    diff = sum(abs(u(:) - u_prev(:))) / (nx * ny * k);
    if diff < tol
        break
    end
    
    fprintf('It %3d: |u-u_prev|=%f\n', it, diff);

end

% visualize result 
[~, classes] = max(u, [], 1);
im_result = reshape(C(classes, :), [ny nx 3]);
imshow([im; im_result]);
